<?php
/*
Plugin Name: EAN Betaling
Plugin URI: http://eriksendesign.dk/
Description: EAN-betaling i WooCommerce
Version: 0.1
Author: Emil Eriksen
Author URI: http://eriksendesign.dk/
License: GPL2
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Check if WooCommerce is active
 **/
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {

    if (!class_exists('WC_EANBetaling')) {
        // Load the main class
        add_action('plugins_loaded', 'wc_eanb_init');

        // Add EAN Betaling as a WooCommerce gateway
        add_filter('woocommerce_payment_gateways', 'wc_eanb_add_gateway');
        
        add_action( 'woocommerce_admin_order_data_after_billing_address', 'eanb_ean_display_admin_order_meta', 10, 1 );
		function eanb_ean_display_admin_order_meta($order){
			if ( get_post_meta( $order->id, 'EAN-nummer', true ) ) echo '<p><strong>EAN-nummer:</strong> ' . get_post_meta( $order->id, 'EAN-nummer', true ) . '</p>';
		}
    }
}

function wc_eanb_init() {

    /**
     * Main class
     */
    class WC_EANBetaling extends WC_Payment_Gateway {
		public function __construct(){
			$this -> id = 'eanb';
			$this->method_title = __( 'EAN Betaling', 'woocommerce' );
			$this->method_description = __( 'EAN-betaling til WooCommerce.', 'woocommerce' );
			$this -> has_fields = true;
			
			$this -> init_form_fields();
			$this -> init_settings();
			
            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->email = $this->get_option('email');
            $this->error = $this->get_option('error');
            $this->receipt = $this->get_option('receipt');
			
			if ( version_compare( WOOCOMMERCE_VERSION, '2.0.0', '>=' ) ) {
			        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( &$this, 'process_admin_options' ) );
			     } else {
			        add_action( 'woocommerce_update_options_payment_gateways', array( &$this, 'process_admin_options' ) );
			}
			
			add_filter('payment_fields', array($this, 'payment_fields'));
			
			add_action( 'woocommerce_email_before_order_table', array( $this, 'email_instructions' ), 10, 2 );
			
		}
		
		function init_form_fields() {
			
			$this -> form_fields = array(
				'enabled' => array(
				    'title' => 'Enable/Disable',
				    'type' => 'checkbox',
				    'label' => 'Enable EAN Betaling',
				    'default' => 'no'),
				'title' => array(
				    'title' => 'Title',
				    'type'=> 'text',
				    'description' => 'This controls the title which the user sees during checkout.',
				    'default' => 'EAN Betaling'),
				'error' => array(
				    'title' => 'Error message',
				    'type'=> 'text',
				    'description' => 'This controls the error message thrown if the entered EAN-number is invalid.',
				    'default' => 'EAN-nummeret skal bestå af 13 cifre og må ikke indeholde bogstaver eller tegn.'),
				'description' => array(
				    'title' => 'Description:',
				    'type' => 'textarea',
				    'description' => 'This controls the description which the user sees during checkout.',
				    'default' => 'Betal med EAN-nummer'),
				'receipt' => array(
				    'title' => 'Text on receipt page:',
				    'type' => 'textarea',
				    'description' => 'This controls the description which the user sees after checkout. "%s" will be replaced with the EAN-number.',
				    'default' => 'Tak for din bestilling! Vi fakturerer EAN-nummeret %s og sender straks din ordre herefter.'),
				'email' => array(
				    'title' => 'Text in email:',
				    'type' => 'textarea',
				    'description' => 'This controls the description in the order confirmation email sent to the user. "%s" will be replaced with the EAN-number.',
				    'default' => 'Du har betalt med følgende EAN-nummer: %s')
			);
		}
			
		public function admin_options() {
			echo '<table class="form-table">';
			// Generate the HTML For the settings form.
			$this -> generate_settings_html();
			echo '</table>';
		
		}
		
		function payment_fields(){
        	if($this -> description) echo wpautop(wptexturize($this -> description));
    	    woocommerce_form_field( 'ean_betaling', array(
		        'type'          => 'text',
		        'placeholder'	=> 'EAN nummer anføres her',
		        'label'         => 'EAN-nummer:'
	        ));
		}
		
		function validate_fields() {
			$ean = $_POST['ean_betaling'];
			if (!$ean || strlen($ean) != 13 || !ctype_digit($ean)) {
				wc_add_notice( wptexturize($this -> error), 'error' );
				return false;
			}
			return true;
		}
		
	    function email_instructions( $order, $sent_to_admin ) {

	    	if ( $sent_to_admin ) return;
	
	    	if ( $order->status !== 'on-hold') return;
	
	    	if ( $order->payment_method !== 'eanb') return;
	
			if($this -> email) {
					$email_text = sprintf( $this -> email, get_post_meta( $order->id, 'EAN-nummer', true ) );
					echo wpautop(wptexturize($email_text));
			}
	    }
		
	    function process_payment( $order_id ) {
	
			$order = new WC_Order( $order_id );
			
			if ($_POST['ean_betaling']) update_post_meta( $order_id, 'EAN-nummer', esc_attr($_POST['ean_betaling']));
	
			// Mark as on-hold (we're awaiting the payment)
			$order->update_status('on-hold', 'Afventer EAN-betaling');
	
			// Reduce stock levels
			$order->reduce_order_stock();
	
			// Remove cart
			WC()->cart->empty_cart();
	
			// Return thankyou redirect
			return array(
				'result' 	=> 'success',
				'redirect'	=> $this->get_return_url( $order )
			);
	    }
    }
}

/**
 * Add EAN Betaling as a gateway to WooCommerce
 */
function wc_eanb_add_gateway($methods) {
    $methods[] = 'WC_EANBetaling';
    return $methods;
}